# backups-lambda
Python scripts per a executar-se com a servei d'AWS Lambda per a crear i esborrar Snapshots de Volums EBS.

[Modificat des del codi de github: https://github.com/cmachler/aws-lambda-ebs-backups]

## Configurant IAM Permissions

Primer hem de crear una política IAM anomenada ebs-backup-worker seguint el document següent:

```
{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Effect": "Allow",
            "Action": [
                "logs:*"
            ],
            "Resource": "arn:aws:logs:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": "ec2:Describe*",
            "Resource": "*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "ec2:CreateSnapshot",
                "ec2:CopySnapshot",
                "ec2:DeleteSnapshot",
                "ec2:CreateTags",
                "ec2:ModifySnapshotAttribute",
                "ec2:ResetSnapshotAttribute"
            ],
            "Resource": [
                "*"
            ]
        },
        {
            "Effect": "Allow",
            "Action": [
                "sns:Publish"
            ],
            "Resource": "*"
        }
    ]
}
```

Continuem creant un role IAM anomenat ebs-backup-worker, seleccionem AWS Lambda com a tipus de rol i li associem la política que acabem de crear.

Es pot comprovar la relació en el rol en "Edit Trust Relationship" deuria semblar a això:

```
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Effect": "Allow",
      "Principal": {
        "Service": "lambda.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}
```

## Afegint AWS Regions a la variable "aws_regions".

AWS Lambda, no admeten valors separats per comes en les variables d’entorn, per això no podem afegir un llistat de regions que volem que passi l’script Python de la Lambda. 
Per a solucionar això, codificarem el llistat de regions en Base64 i en el codi font, el descodifiquem i fem un Split per les comes, així aconseguim una llista d’elements

Utilitzem Python per aconseguir el Base64 del llistat de les regions:

```
Python 3.8.0 (tags/v3.8.0:fa919fd, Oct 14 2019, 19:21:23) [MSC v.1916 32 bit (Intel)] on win32
Type "help", "copyright", "credits" or "license" for more information.
>>> import base64
>>> encoded = base64.b64encode(b'eu-west-1')
>>> encoded
b'ZXUtd2VzdC0x'
>>> data = base64.b64decode(encoded)
>>> data
b'eu-west-1'
>>>
```

**En les variables d’entorn de la lambda hem de crear una variable anomenada aws_regions amb el valor ZXUtd2VzdC0x que ens ha donat com a resultat l’script de Python.**

## Si es vol publicar mitjançant SNS, afegir la variable "aws_sns_arn"

ÉS una variable d'entorn opcional. Si es vol publicar en SNS, s'ha d'afegir la variable amb valor l'arn de la publicació.

## Creant les funcions Lambda

Hem de crear dues AWS Lambda. Una per fer funcionar l’script que realitza els snapshots i una altra és un script de neteja. 
Escollim com a llenguatge Python 3.7 i com a rol d’execució el rol ebs-backup-worker que hem creat en aquest mateix punt.
Creem un trigger, del tipus CloudWatch Events i amb una expressió de programació que és un cron. L’expressió serà cron(0 6 ? * MON *) és a dir, tots els dilluns a les 6:00h. s’executarà.

De la mateixa manera que s’ha descrit, hem de crear un altre AWS Lambda, aquesta anomenada backupCleanupReserves que contindrà el script de neteja. El cron en aquesta Lambda seria de posterior execució a l’anterior: cron(0 7 ? * MON *)

## Tagging your EC2 instances to backup

Ara hem de crear tags en les instancies EC2 sobre les que volem que actuï les Lambdes. 

| Tag Key           | Tag Value                           | Notes |
| -------------     |:-------------:                      | -----:|
| Backup            |                                     | Value Not Needed |
| Retention         | *Number of Days to Retain Snapshot* |  Default is 7 Days|
|Skip_Backup_Volumes|*volume id(s) in CSV string*         |    List either a single volume-id, or multiple volumes-ids in a Comma Separated Value String |

